@@ -1,5 +1,5 @@
 /*
-#             (C) 2008 Hans de Goede <hdegoede@redhat.com>
+#             (C) 2008-2011 Hans de Goede <hdegoede@redhat.com>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +13,10 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
+#include <config.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
@@ -28,47 +29,105 @@
 
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 
-/* Note for proper functioning of v4lconvert_enum_fmt the first entries in
-   supported_src_pixfmts must match with the entries in supported_dst_pixfmts */
-#define SUPPORTED_DST_PIXFMTS \
-{ V4L2_PIX_FMT_RGB24,  0 }, \
-{ V4L2_PIX_FMT_BGR24,  0 }, \
-{ V4L2_PIX_FMT_YUV420, 0 }, \
-{ V4L2_PIX_FMT_YVU420, 0 }
+static void *dev_init(int fd)
+{
+	return NULL;
+}
+
+static void dev_close(void *dev_ops_priv)
+{
+}
+
+static int dev_ioctl(void *dev_ops_priv, int fd, unsigned long cmd, void *arg)
+{
+	return SYS_IOCTL(fd, cmd, arg);
+}
+
+static ssize_t dev_read(void *dev_ops_priv, int fd, void *buf, size_t len)
+{
+	return SYS_READ(fd, buf, len);
+}
+
+static ssize_t dev_write(void *dev_ops_priv, int fd, const void *buf,
+                         size_t len)
+{
+	return SYS_WRITE(fd, buf, len);
+}
+
+static const struct libv4l_dev_ops default_dev_ops = {
+	.init = dev_init,
+	.close = dev_close,
+	.ioctl = dev_ioctl,
+	.read = dev_read,
+	.write = dev_write,
+};
+
+const struct libv4l_dev_ops *v4lconvert_get_default_dev_ops()
+{
+	return &default_dev_ops;
+}
 
 static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
 		unsigned int pixelformat, int index);
 
-/* Note uncompressed formats must go first so that they are prefered by
-   v4lconvert_try_format for low resolutions */
+/* Note for proper functioning of v4lconvert_enum_fmt the first entries in
+   supported_src_pixfmts must match with the entries in supported_dst_pixfmts */
+#define SUPPORTED_DST_PIXFMTS \
+	/* fourcc			bpp	rgb	yuv	needs      */ \
+	/*					rank	rank	conversion */ \
+	{ V4L2_PIX_FMT_RGB24,		24,	 1,	 5,	0 }, \
+	{ V4L2_PIX_FMT_BGR24,		24,	 1,	 5,	0 }, \
+	{ V4L2_PIX_FMT_YUV420,		12,	 6,	 1,	0 }, \
+	{ V4L2_PIX_FMT_YVU420,		12,	 6,	 1,	0 }
+
 static const struct v4lconvert_pixfmt supported_src_pixfmts[] = {
 	SUPPORTED_DST_PIXFMTS,
-	{ V4L2_PIX_FMT_YUYV,         0 },
-	{ V4L2_PIX_FMT_YVYU,         0 },
-	{ V4L2_PIX_FMT_UYVY,         0 },
-	{ V4L2_PIX_FMT_RGB565,       0 },
-	{ V4L2_PIX_FMT_SN9C20X_I420, V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SBGGR8,       V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SGBRG8,       V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SGRBG8,       V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SRGGB8,       V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_STV0680,      V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SPCA501,      V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SPCA505,      V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SPCA508,      V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_CPIA1,        V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_HM12,         V4LCONVERT_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_MJPEG,        V4LCONVERT_COMPRESSED },
-	{ V4L2_PIX_FMT_JPEG,         V4LCONVERT_COMPRESSED },
-	{ V4L2_PIX_FMT_SPCA561,      V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SN9C10X,      V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SN9C2028,     V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_PAC207,       V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_MR97310A,     V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_SQ905C,       V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_PJPG,         V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_OV511,        V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
-	{ V4L2_PIX_FMT_OV518,        V4LCONVERT_COMPRESSED_AND_NEEDS_CONVERSION },
+	/* packed rgb formats */
+	{ V4L2_PIX_FMT_RGB565,		16,	 4,	 6,	0 },
+	/* yuv 4:2:2 formats */
+	{ V4L2_PIX_FMT_YUYV,		16,	 5,	 4,	0 },
+	{ V4L2_PIX_FMT_YVYU,		16,	 5,	 4,	0 },
+	{ V4L2_PIX_FMT_UYVY,		16,	 5,	 4,	0 },
+	/* yuv 4:2:0 formats */
+	{ V4L2_PIX_FMT_SPCA501,		12,      6,	 3,	1 },
+	{ V4L2_PIX_FMT_SPCA505,		12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_SPCA508,		12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_CIT_YYVYUY,	12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_KONICA420,	12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_SN9C20X_I420,	12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_M420,		12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_HM12,		12,	 6,	 3,	1 },
+	{ V4L2_PIX_FMT_CPIA1,		 0,	 6,	 3,	1 },
+	/* JPEG and variants */
+	{ V4L2_PIX_FMT_MJPEG,		 0,	 7,	 7,	0 },
+	{ V4L2_PIX_FMT_JPEG,		 0,	 7,	 7,	0 },
+	{ V4L2_PIX_FMT_PJPG,		 0,	 7,	 7,	1 },
+	{ V4L2_PIX_FMT_JPGL,		 0,	 7,	 7,	1 },
+	{ V4L2_PIX_FMT_OV511,		 0,	 7,	 7,	1 },
+	{ V4L2_PIX_FMT_OV518,		 0,	 7,	 7,	1 },
+	/* uncompressed bayer */
+	{ V4L2_PIX_FMT_SBGGR8,		 8,	 8,	 8,	1 },
+	{ V4L2_PIX_FMT_SGBRG8,		 8,	 8,	 8,	1 },
+	{ V4L2_PIX_FMT_SGRBG8,		 8,	 8,	 8,	1 },
+	{ V4L2_PIX_FMT_SRGGB8,		 8,	 8,	 8,	1 },
+	{ V4L2_PIX_FMT_STV0680,		 8,	 8,	 8,	1 },
+	/* compressed bayer */
+	{ V4L2_PIX_FMT_SPCA561,		 0,	 9,	 9,	1 },
+	{ V4L2_PIX_FMT_SN9C10X,		 0,	 9,	 9,	1 },
+	{ V4L2_PIX_FMT_SN9C2028,	 0,	 9,	 9,	1 },
+	{ V4L2_PIX_FMT_PAC207,		 0,	 9,	 9,	1 },
+	{ V4L2_PIX_FMT_MR97310A,	 0,	 9,	 9,	1 },
+#ifdef HAVE_JPEG
+	{ V4L2_PIX_FMT_JL2005BCD,	 0,	 9,	 9,	1 },
+#endif
+	{ V4L2_PIX_FMT_SQ905C,		 0,	 9,	 9,	1 },
+	/* special */
+	{ V4L2_PIX_FMT_SE401,		 0,	 8,	 9,	1 },
+	/* grey formats */
+	{ V4L2_PIX_FMT_GREY,		 8,	20,	20,	0 },
+	{ V4L2_PIX_FMT_Y4,		 8,	20,	20,	0 },
+	{ V4L2_PIX_FMT_Y6,		 8,	20,	20,	0 },
+	{ V4L2_PIX_FMT_Y10BPACK,	10,	20,	20,	0 },
 };
 
 static const struct v4lconvert_pixfmt supported_dst_pixfmts[] = {
@@ -89,6 +148,12 @@ static const int v4lconvert_crop_res[][2] = {
 };
 
 struct v4lconvert_data *v4lconvert_create(int fd)
+{
+	return v4lconvert_create_with_dev_ops(fd, NULL, &default_dev_ops); 
+}
+
+struct v4lconvert_data *v4lconvert_create_with_dev_ops(int fd, void *dev_ops_priv,
+		const struct libv4l_dev_ops *dev_ops)
 {
 	int i, j;
 	struct v4lconvert_data *data = calloc(1, sizeof(struct v4lconvert_data));
@@ -104,7 +169,10 @@ struct v4lconvert_data *v4lconvert_create(int fd)
 	}
 
 	data->fd = fd;
+	data->dev_ops = dev_ops;
+	data->dev_ops_priv = dev_ops_priv;
 	data->decompress_pid = -1;
+	data->fps = 30;
 
 	/* Check supported formats */
 	for (i = 0; ; i++) {
@@ -112,41 +180,47 @@ struct v4lconvert_data *v4lconvert_create(int fd)
 
 		fmt.index = i;
 
-		if (SYS_IOCTL(data->fd, VIDIOC_ENUM_FMT, &fmt))
+		if (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_ENUM_FMT, &fmt))
 			break;
 
 		for (j = 0; j < ARRAY_SIZE(supported_src_pixfmts); j++)
-			if (fmt.pixelformat == supported_src_pixfmts[j].fmt) {
-				data->supported_src_formats |= 1 << j;
-				v4lconvert_get_framesizes(data, fmt.pixelformat, j);
-				if (!(supported_src_pixfmts[j].flags & V4LCONVERT_NEEDS_CONVERSION))
-					always_needs_conversion = 0;
+			if (fmt.pixelformat == supported_src_pixfmts[j].fmt)
 				break;
-			}
 
-		if (j == ARRAY_SIZE(supported_src_pixfmts))
+		if (j < ARRAY_SIZE(supported_src_pixfmts)) {
+			data->supported_src_formats |= 1ULL << j;
+			v4lconvert_get_framesizes(data, fmt.pixelformat, j);
+			if (!supported_src_pixfmts[j].needs_conversion)
+				always_needs_conversion = 0;
+		} else
 			always_needs_conversion = 0;
 	}
 
 	data->no_formats = i;
 
 	/* Check if this cam has any special flags */
-	if (SYS_IOCTL(data->fd, VIDIOC_QUERYCAP, &cap) == 0) {
+	if (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_QUERYCAP, &cap) == 0) {
 		if (!strcmp((char *)cap.driver, "uvcvideo"))
 			data->flags |= V4LCONVERT_IS_UVC;
-		else if (!strcmp((char *)cap.driver, "sn9c20x"))
-			data->flags |= V4LCONVERT_IS_SN9C20X;
 
+		if (cap.capabilities & V4L2_CAP_DEVICE_CAPS)
+			cap.capabilities = cap.device_caps;
 		if ((cap.capabilities & 0xff) & ~V4L2_CAP_VIDEO_CAPTURE)
 			always_needs_conversion = 0;
 	}
 
-	data->control = v4lcontrol_create(fd, always_needs_conversion);
+	data->control = v4lcontrol_create(fd, dev_ops_priv, dev_ops,
+						always_needs_conversion);
 	if (!data->control) {
 		free(data);
 		return NULL;
 	}
+	data->bandwidth = v4lcontrol_get_bandwidth(data->control);
 	data->control_flags = v4lcontrol_get_flags(data->control);
+	if (data->control_flags & V4LCONTROL_FORCE_TINYJPEG)
+		data->flags |= V4LCONVERT_USE_TINYJPEG;
 
 	data->processing = v4lprocessing_create(fd, data->control);
 	if (!data->processing) {
@@ -160,14 +234,21 @@ struct v4lconvert_data *v4lconvert_create(int fd)
 
 void v4lconvert_destroy(struct v4lconvert_data *data)
 {
+	if (!data)
+		return;
+
 	v4lprocessing_destroy(data->processing);
 	v4lcontrol_destroy(data->control);
-	if (data->jdec) {
+	if (data->tinyjpeg) {
 		unsigned char *comps[3] = { NULL, NULL, NULL };
 
-		tinyjpeg_set_components(data->jdec, comps, 3);
-		tinyjpeg_free(data->jdec);
+		tinyjpeg_set_components(data->tinyjpeg, comps, 3);
+		tinyjpeg_free(data->tinyjpeg);
 	}
+#ifdef HAVE_JPEG
+	if (data->cinfo_initialized)
+		jpeg_destroy_decompress(&data->cinfo);
+#endif // HAVE_JPEG
 	v4lconvert_helper_cleanup(data);
 	free(data->convert1_buf);
 	free(data->convert2_buf);
@@ -204,11 +285,12 @@ int v4lconvert_enum_fmt(struct v4lconvert_data *data, struct v4l2_fmtdesc *fmt)
 	if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
 			(!v4lconvert_supported_dst_fmt_only(data) &&
 			 fmt->index < data->no_formats))
-		return SYS_IOCTL(data->fd, VIDIOC_ENUM_FMT, fmt);
+		return data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_ENUM_FMT, fmt);
 
 	for (i = 0; i < ARRAY_SIZE(supported_dst_pixfmts); i++)
 		if (v4lconvert_supported_dst_fmt_only(data) ||
-				!(data->supported_src_formats & (1 << i))) {
+				!(data->supported_src_formats & (1ULL << i))) {
 			faked_fmts[no_faked_fmts] = supported_dst_pixfmts[i].fmt;
 			no_faked_fmts++;
 		}
@@ -230,11 +312,63 @@ int v4lconvert_enum_fmt(struct v4lconvert_data *data, struct v4l2_fmtdesc *fmt)
 	fmt->description[2] = (faked_fmts[i] >> 16) & 0xff;
 	fmt->description[3] = faked_fmts[i] >> 24;
 	fmt->description[4] = '\0';
-	memset(fmt->reserved, 0, 4);
+	memset(fmt->reserved, 0, sizeof(fmt->reserved));
 
 	return 0;
 }
 
+/* This function returns a value to rank (sort) source format by preference
+   when multiple source formats are available for a certain resolution, the
+   source format for which this function returns the lowest value wins.
+   
+   This function uses the rgb_rank resp. yuv_rank values as a base when
+   converting to rgb32 resp. yuv420. The initial ranks range from 1 - 10,
+   the initial rank purely expresses the CPU cost of doing the conversion, the
+   ranking algorithm will give a penalty of 10 points if
+   (width * height * fps * bpp / 8) > bandwidth
+   thus disqualifying a src format which causes the bandwidth to be exceeded,
+   except when all of them cause this.
+   
+   Note grey scale formats start at 20 rather then 1-10, because we want to
+   never autoselect them, unless they are the only choice */
+static int v4lconvert_get_rank(struct v4lconvert_data *data,
+	int src_index, int src_width, int src_height,
+	unsigned int dest_pixelformat)
+{
+	int needed, rank = 0;
+
+	switch (dest_pixelformat) {
+	case V4L2_PIX_FMT_RGB24:
+	case V4L2_PIX_FMT_BGR24:
+		rank = supported_src_pixfmts[src_index].rgb_rank;
+		break;
+	case V4L2_PIX_FMT_YUV420:
+	case V4L2_PIX_FMT_YVU420:
+		rank = supported_src_pixfmts[src_index].yuv_rank;
+		break;
+	}
+
+	/* So that if both rgb32 and bgr32 are supported, or both yuv420 and
+	   yvu420 the right one wins */
+	if (supported_src_pixfmts[src_index].fmt == dest_pixelformat)
+		rank--;
+
+	/* check bandwidth needed */
+	needed = src_width * src_height * data->fps *
+		 supported_src_pixfmts[src_index].bpp / 8;
+	if (data->bandwidth && needed > data->bandwidth)
+		rank += 10;
+#if 0
+	printf("ranked: %c%c%c%c for %dx%d @ %d fps, needed: %d, bandwidth: %d, rank: %d\n",
+	       supported_src_pixfmts[src_index].fmt & 0xff,
+	       (supported_src_pixfmts[src_index].fmt >> 8) & 0xff,
+	       (supported_src_pixfmts[src_index].fmt >> 16) & 0xff,
+	       supported_src_pixfmts[src_index].fmt >> 24, src_width,
+	       src_height, data->fps, needed, data->bandwidth, rank);
+#endif
+	return rank;
+}
+
 /* Find out what format to use based on the (cached) results of enum
    framesizes instead of doing a zillion try_fmt calls. This function
    currently is intended for use with UVC cams only. This is esp.
@@ -242,10 +376,11 @@ int v4lconvert_enum_fmt(struct v4lconvert_data *data, struct v4l2_fmtdesc *fmt)
 static int v4lconvert_do_try_format_uvc(struct v4lconvert_data *data,
 		struct v4l2_format *dest_fmt, struct v4l2_format *src_fmt)
 {
-	int i;
+	int i, rank;
 	unsigned int closest_fmt_size_diff = -1;
 	int best_framesize = 0;/* Just use the first format if no small enough one */
 	int best_format = 0;
+	int best_rank = 100;
 
 	for (i = 0; i < data->no_framesizes; i++) {
 		if (data->framesizes[i].discrete.width <= dest_fmt->fmt.pix.width &&
@@ -269,12 +404,16 @@ static int v4lconvert_do_try_format_uvc(struct v4lconvert_data *data,
 		if (!(data->framesizes[best_framesize].pixel_format & (1 << i)))
 			continue;
 
-		if (!best_format ||
-				supported_src_pixfmts[i].fmt == dest_fmt->fmt.pix.pixelformat ||
-				((data->framesizes[best_framesize].discrete.width > 180 ||
-				  data->framesizes[best_framesize].discrete.height > 148) &&
-				 (supported_src_pixfmts[i].flags & V4LCONVERT_COMPRESSED)))
+		/* Note the hardcoded use of discrete is based on this function
+		   only getting called for uvc devices */
+		rank = v4lconvert_get_rank(data, i,
+			    data->framesizes[best_framesize].discrete.width,
+			    data->framesizes[best_framesize].discrete.height,
+			    dest_fmt->fmt.pix.pixelformat);
+		if (rank < best_rank) {
+			best_rank = rank;
 			best_format = supported_src_pixfmts[i].fmt;
+		}
 	}
 
 	dest_fmt->fmt.pix.width = data->framesizes[best_framesize].discrete.width;
@@ -296,8 +435,8 @@ static int v4lconvert_do_try_format_uvc(struct v4lconvert_data *data,
 static int v4lconvert_do_try_format(struct v4lconvert_data *data,
 		struct v4l2_format *dest_fmt, struct v4l2_format *src_fmt)
 {
-	int i;
-	unsigned int closest_fmt_size_diff = -1;
+	int i, size_x_diff, size_y_diff, rank, best_rank = 0;
+	unsigned int size_diff, closest_fmt_size_diff = -1;
 	unsigned int desired_pixfmt = dest_fmt->fmt.pix.pixelformat;
 	struct v4l2_format try_fmt, closest_fmt = { .type = 0 };
 
@@ -306,30 +445,36 @@ static int v4lconvert_do_try_format(struct v4lconvert_data *data,
 
 	for (i = 0; i < ARRAY_SIZE(supported_src_pixfmts); i++) {
 		/* is this format supported? */
-		if (!(data->supported_src_formats & (1 << i)))
+		if (!(data->supported_src_formats & (1ULL << i)))
 			continue;
 
 		try_fmt = *dest_fmt;
 		try_fmt.fmt.pix.pixelformat = supported_src_pixfmts[i].fmt;
+		if (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_TRY_FMT, &try_fmt))
+			continue;
 
-		if (!SYS_IOCTL(data->fd, VIDIOC_TRY_FMT, &try_fmt)) {
-			if (try_fmt.fmt.pix.pixelformat == supported_src_pixfmts[i].fmt) {
-				int size_x_diff = abs((int)try_fmt.fmt.pix.width -
-						(int)dest_fmt->fmt.pix.width);
-				int size_y_diff = abs((int)try_fmt.fmt.pix.height -
-						(int)dest_fmt->fmt.pix.height);
-				unsigned int size_diff = size_x_diff * size_x_diff +
-					size_y_diff * size_y_diff;
+		if (try_fmt.fmt.pix.pixelformat !=
+		    supported_src_pixfmts[i].fmt)
+			continue;
 
-				if (size_diff < closest_fmt_size_diff ||
-						(size_diff == closest_fmt_size_diff &&
-						 (supported_src_pixfmts[i].fmt == desired_pixfmt ||
-						  ((try_fmt.fmt.pix.width > 180 || try_fmt.fmt.pix.height > 148) &&
-						   (supported_src_pixfmts[i].flags & V4LCONVERT_COMPRESSED))))) {
-					closest_fmt_size_diff = size_diff;
-					closest_fmt = try_fmt;
-				}
-			}
+		/* Did we get a better match than before? */
+		size_x_diff = (int)try_fmt.fmt.pix.width -
+			      (int)dest_fmt->fmt.pix.width;
+		size_y_diff = (int)try_fmt.fmt.pix.height -
+			      (int)dest_fmt->fmt.pix.height;
+		size_diff = size_x_diff * size_x_diff +
+			    size_y_diff * size_y_diff;
+
+		rank = v4lconvert_get_rank(data, i,
+					   try_fmt.fmt.pix.width,
+					   try_fmt.fmt.pix.height,
+					   desired_pixfmt);
+		if (size_diff < closest_fmt_size_diff ||
+		    (size_diff == closest_fmt_size_diff && rank < best_rank)) {
+			closest_fmt = try_fmt;
+			closest_fmt_size_diff = size_diff;
+			best_rank = rank;
 		}
 	}
 
@@ -380,7 +525,8 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
 	if (!v4lconvert_supported_dst_format(dest_fmt->fmt.pix.pixelformat) ||
 			dest_fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
 			v4lconvert_do_try_format(data, &try_dest, &try_src)) {
-		result = SYS_IOCTL(data->fd, VIDIOC_TRY_FMT, dest_fmt);
+		result = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_TRY_FMT, dest_fmt);
 		if (src_fmt)
 			*src_fmt = *dest_fmt;
 		return result;
@@ -389,7 +535,7 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
 	/* In case of a non exact resolution match, try again with a slightly larger
 	   resolution as some weird devices are not able to crop of the number of
 	   extra (border) pixels most sensors have compared to standard resolutions,
-	   which we will then just crop of in software */
+	   which we will then just crop off in software */
 	if (try_dest.fmt.pix.width != desired_width ||
 			try_dest.fmt.pix.height != desired_height) {
 		try2_dest = *dest_fmt;
@@ -498,6 +644,9 @@ static int v4lconvert_processing_needs_double_conversion(
 	case V4L2_PIX_FMT_SN9C10X:
 	case V4L2_PIX_FMT_PAC207:
 	case V4L2_PIX_FMT_MR97310A:
+#ifdef HAVE_JPEG
+	case V4L2_PIX_FMT_JL2005BCD:
+#endif
 	case V4L2_PIX_FMT_SN9C2028:
 	case V4L2_PIX_FMT_SQ905C:
 	case V4L2_PIX_FMT_SBGGR8:
@@ -531,7 +680,7 @@ unsigned char *v4lconvert_alloc_buffer(int needed,
 	return *buf;
 }
 
-static int v4lconvert_oom_error(struct v4lconvert_data *data)
+int v4lconvert_oom_error(struct v4lconvert_data *data)
 {
 	V4LCONVERT_ERR("could not allocate memory\n");
 	errno = ENOMEM;
@@ -539,113 +688,59 @@ static int v4lconvert_oom_error(struct v4lconvert_data *data)
 }
 
 static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
-		unsigned char *src, int src_size, unsigned char *dest, int dest_size,
-		struct v4l2_format *fmt, unsigned int dest_pix_fmt)
+	unsigned char *src, int src_size, unsigned char *dest, int dest_size,
+	struct v4l2_format *fmt, unsigned int dest_pix_fmt)
 {
-	unsigned int header_width, header_height;
-	int result = 0, jpeg_flags = TINYJPEG_FLAGS_MJPEG_TABLE;
-	unsigned char *components[3];
+	int result = 0;
 	unsigned int src_pix_fmt = fmt->fmt.pix.pixelformat;
 	unsigned int width  = fmt->fmt.pix.width;
 	unsigned int height = fmt->fmt.pix.height;
+	unsigned int bytesperline = fmt->fmt.pix.bytesperline;
 
 	switch (src_pix_fmt) {
-	case V4L2_PIX_FMT_PJPG:
-		jpeg_flags |= TINYJPEG_FLAGS_PIXART_JPEG;
-		/* Fall through */
+	/* JPG and variants */
 	case V4L2_PIX_FMT_MJPEG:
 	case V4L2_PIX_FMT_JPEG:
-		if (!data->jdec) {
-			data->jdec = tinyjpeg_init();
-			if (!data->jdec)
-				return v4lconvert_oom_error(data);
-		}
-		tinyjpeg_set_flags(data->jdec, jpeg_flags);
-		if (tinyjpeg_parse_header(data->jdec, src, src_size)) {
-			V4LCONVERT_ERR("parsing JPEG header: %s",
-					tinyjpeg_get_errorstring(data->jdec));
-			errno = EIO;
-			return -1;
-		}
-		tinyjpeg_get_size(data->jdec, &header_width, &header_height);
-
-		if (header_width != width || header_height != height) {
-			/* Check for (pixart) rotated JPEG */
-			if (header_width == height && header_height == width) {
-				if (!(data->control_flags & V4LCONTROL_ROTATED_90_JPEG)) {
-					V4LCONVERT_ERR("JPEG needs 90° rotation, please report "
-							"this to <hdegoede@redhat.com>\n");
-					errno = EIO;
-					return -1;
-				}
-				fmt->fmt.pix.width = header_width;
-				fmt->fmt.pix.height = header_height;
-			} else {
-				V4LCONVERT_ERR("unexpected width / height in JPEG header"
-						"expected: %ux%u, header: %ux%u\n", width, height,
-						header_width, header_height);
-				errno = EIO;
-				return -1;
-			}
-		} else if ((data->control_flags & V4LCONTROL_ROTATED_90_JPEG)) {
-			fprintf(stderr, "libv4lconvert: expected 90° rotated JPEG, but got "
-					"normal JPEG, please report this to <hdegoede@redhat.com>\n");
-			V4LCONVERT_ERR("expected 90° rotated JPEG, but got normal JPEG\n");
-			errno = EAGAIN;
-			data->control_flags &= ~V4LCONTROL_ROTATED_90_JPEG;
-			return -1;
-		}
-
-		components[0] = dest;
-
-		switch (dest_pix_fmt) {
-		case V4L2_PIX_FMT_RGB24:
-			tinyjpeg_set_components(data->jdec, components, 1);
-			result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_RGB24);
-			break;
-		case V4L2_PIX_FMT_BGR24:
-			tinyjpeg_set_components(data->jdec, components, 1);
-			result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_BGR24);
-			break;
-		case V4L2_PIX_FMT_YUV420:
-			components[1] = components[0] + width * height;
-			components[2] = components[1] + width * height / 4;
-			tinyjpeg_set_components(data->jdec, components, 3);
-			result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_YUV420P);
-			break;
-		case V4L2_PIX_FMT_YVU420:
-			components[2] = components[0] + width * height;
-			components[1] = components[2] + width * height / 4;
-			tinyjpeg_set_components(data->jdec, components, 3);
-			result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_YUV420P);
-			break;
-		}
-
-		if (result) {
-			/* Pixart webcam's seem to regulary generate corrupt frames, which
-			   are best thrown away to avoid flashes in the video stream. Tell
-			   the upper layer this is an intermediate fault and it should try
-			   again with a new buffer by setting errno to EAGAIN */
-			if (src_pix_fmt == V4L2_PIX_FMT_PJPG ||
-					data->flags & V4LCONVERT_IS_SN9C20X) {
-				V4LCONVERT_ERR("decompressing JPEG: %s",
-						tinyjpeg_get_errorstring(data->jdec));
-				errno = EAGAIN;
-				return -1;
-			} else {
-				/* If the JPEG header checked out ok and we get an error during actual
-				   decompression, log the error, but don't return an errorcode to the
-				   application, so that the user gets what we managed to decompress */
-				fprintf(stderr, "libv4lconvert: Error decompressing JPEG: %s",
-						tinyjpeg_get_errorstring(data->jdec));
+#ifdef HAVE_JPEG
+		if (data->flags & V4LCONVERT_USE_TINYJPEG) {
+#endif // HAVE_JPEG
+			result = v4lconvert_decode_jpeg_tinyjpeg(data,
+							src, src_size, dest,
+							fmt, dest_pix_fmt, 0);
+#ifdef HAVE_JPEG
+		} else {
+			result = v4lconvert_decode_jpeg_libjpeg(data,
+							src, src_size, dest,
+							fmt, dest_pix_fmt);
+			if (result == -1 && errno == EOPNOTSUPP) {
+				/* Fall back to tinyjpeg */
+				jpeg_destroy_decompress(&data->cinfo);
+				data->cinfo_initialized = 0;
+				data->flags |= V4LCONVERT_USE_TINYJPEG;
+				result = v4lconvert_decode_jpeg_tinyjpeg(data,
+							src, src_size, dest,
+							fmt, dest_pix_fmt, 0);
 			}
 		}
+#endif // HAVE_JPEG
+		break;
+	case V4L2_PIX_FMT_PJPG:
+		result = v4lconvert_decode_jpeg_tinyjpeg(data, src, src_size,
+				dest, fmt, dest_pix_fmt,
+				TINYJPEG_FLAGS_PIXART_JPEG);
+		break;
+	case V4L2_PIX_FMT_JPGL:
+		result = v4lconvert_decode_jpgl(src, src_size, dest_pix_fmt,
+						dest, width, height);
 		break;
 
-		/* Custom cam specific YUV formats */
+	/* Custom cam specific YUV formats */
 	case V4L2_PIX_FMT_SPCA501:
 	case V4L2_PIX_FMT_SPCA505:
 	case V4L2_PIX_FMT_SPCA508:
+	case V4L2_PIX_FMT_CIT_YYVYUY:
+	case V4L2_PIX_FMT_KONICA420:
+	case V4L2_PIX_FMT_M420:
 	case V4L2_PIX_FMT_SN9C20X_I420:
 	case V4L2_PIX_FMT_CPIA1:
 	case V4L2_PIX_FMT_OV511:
@@ -679,6 +774,15 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		case V4L2_PIX_FMT_SPCA508:
 			v4lconvert_spca508_to_yuv420(src, d, width, height, yvu);
 			break;
+		case V4L2_PIX_FMT_CIT_YYVYUY:
+			v4lconvert_cit_yyvyuy_to_yuv420(src, d, width, height, yvu);
+			break;
+		case V4L2_PIX_FMT_KONICA420:
+			v4lconvert_konica_yuv420_to_yuv420(src, d, width, height, yvu);
+			break;
+		case V4L2_PIX_FMT_M420:
+			v4lconvert_m420_to_yuv420(src, d, width, height, yvu);
+			break;
 		case V4L2_PIX_FMT_SN9C20X_I420:
 			v4lconvert_sn9c20x_to_yuv420(src, d, width, height, yvu);
 			break;
@@ -691,7 +795,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			}
 			break;
 		case V4L2_PIX_FMT_OV511:
-			if (v4lconvert_helper_decompress(data, LIBDIR "/" LIBSUBDIR "/ov511-decomp",
+			if (v4lconvert_helper_decompress(data, LIBV4LCONVERT_PRIV_DIR "/ov511-decomp",
 						src, src_size, d, d_size, width, height, yvu)) {
 				/* Corrupt frame, better get another one */
 				errno = EAGAIN;
@@ -699,7 +803,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			}
 			break;
 		case V4L2_PIX_FMT_OV518:
-			if (v4lconvert_helper_decompress(data, LIBDIR "/" LIBSUBDIR "/ov518-decomp",
+			if (v4lconvert_helper_decompress(data, LIBV4LCONVERT_PRIV_DIR "/ov518-decomp",
 						src, src_size, d, d_size, width, height, yvu)) {
 				/* Corrupt frame, better get another one */
 				errno = EAGAIN;
@@ -744,6 +848,9 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	case V4L2_PIX_FMT_SN9C10X:
 	case V4L2_PIX_FMT_PAC207:
 	case V4L2_PIX_FMT_MR97310A:
+#ifdef HAVE_JPEG
+	case V4L2_PIX_FMT_JL2005BCD:
+#endif
 	case V4L2_PIX_FMT_SN9C2028:
 	case V4L2_PIX_FMT_SQ905C:
 	case V4L2_PIX_FMT_STV0680: { /* Not compressed but needs some shuffling */
@@ -782,6 +889,18 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			}
 			tmpfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
 			break;
+#ifdef HAVE_JPEG
+		case V4L2_PIX_FMT_JL2005BCD:
+			if (v4lconvert_decode_jl2005bcd(data, src, src_size,
+							tmpbuf,
+							width, height)) {
+				/* Corrupt frame, better get another one */
+				errno = EAGAIN;
+				return -1;
+			}
+			tmpfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SRGGB8;
+			break;
+#endif
 		case V4L2_PIX_FMT_SN9C2028:
 			v4lconvert_decode_sn9c2028(src, tmpbuf, width, height);
 			tmpfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
@@ -803,6 +922,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		/* Deliberate fall through to raw bayer fmt code! */
 		src_pix_fmt = tmpfmt.fmt.pix.pixelformat;
 		src = tmpbuf;
+		src_size = width * height;
 		/* fall through */
 	}
 
@@ -813,18 +933,100 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	case V4L2_PIX_FMT_SRGGB8:
 		switch (dest_pix_fmt) {
 		case V4L2_PIX_FMT_RGB24:
-			v4lconvert_bayer_to_rgb24(src, dest, width, height, src_pix_fmt);
+			v4lconvert_bayer_to_rgb24(src, dest, width, height, bytesperline, src_pix_fmt);
 			break;
 		case V4L2_PIX_FMT_BGR24:
-			v4lconvert_bayer_to_bgr24(src, dest, width, height, src_pix_fmt);
+			v4lconvert_bayer_to_bgr24(src, dest, width, height, bytesperline, src_pix_fmt);
 			break;
 		case V4L2_PIX_FMT_YUV420:
-			v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 0);
+			v4lconvert_bayer_to_yuv420(src, dest, width, height, bytesperline, src_pix_fmt, 0);
 			break;
 		case V4L2_PIX_FMT_YVU420:
-			v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 1);
+			v4lconvert_bayer_to_yuv420(src, dest, width, height, bytesperline, src_pix_fmt, 1);
 			break;
 		}
+		if (src_size < (width * height)) {
+			V4LCONVERT_ERR("short raw bayer data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
+		break;
+
+	case V4L2_PIX_FMT_SE401: {
+		unsigned char *d = NULL;
+
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+			d = dest;
+			break;
+		case V4L2_PIX_FMT_BGR24:
+		case V4L2_PIX_FMT_YUV420:
+		case V4L2_PIX_FMT_YVU420:
+			d = v4lconvert_alloc_buffer(width * height * 3,
+					&data->convert_pixfmt_buf,
+					&data->convert_pixfmt_buf_size);
+			if (!d)
+				return v4lconvert_oom_error(data);
+
+			fmt->fmt.pix.pixelformat = V4L2_PIX_FMT_RGB24;
+			v4lconvert_fixup_fmt(fmt);
+			break;
+		}
+
+		result = v4lconvert_se401_to_rgb24(data, src, src_size, d,
+						   width, height);
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_BGR24:
+			v4lconvert_swap_rgb(d, dest, width, height);
+			break;
+		case V4L2_PIX_FMT_YUV420:
+			v4lconvert_rgb24_to_yuv420(d, dest, fmt, 0, 0);
+			break;
+		case V4L2_PIX_FMT_YVU420:
+			v4lconvert_rgb24_to_yuv420(d, dest, fmt, 0, 1);
+			break;
+		}
+		break;
+	}
+
+	case V4L2_PIX_FMT_GREY:
+	case V4L2_PIX_FMT_Y4:
+	case V4L2_PIX_FMT_Y6:
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+	        case V4L2_PIX_FMT_BGR24:
+			v4lconvert_grey_to_rgb24(src, dest, width, height);
+			break;
+		case V4L2_PIX_FMT_YUV420:
+		case V4L2_PIX_FMT_YVU420:
+			v4lconvert_grey_to_yuv420(src, dest, fmt);
+			break;
+		}
+		if (src_size < (width * height)) {
+			V4LCONVERT_ERR("short grey data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
+		break;
+
+	case V4L2_PIX_FMT_Y10BPACK:
+		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+	        case V4L2_PIX_FMT_BGR24:
+			result = v4lconvert_y10b_to_rgb24(data, src, dest,
+							  width, height);
+			break;
+		case V4L2_PIX_FMT_YUV420:
+		case V4L2_PIX_FMT_YVU420:
+			result = v4lconvert_y10b_to_yuv420(data, src, dest,
+							   width, height);
+			break;
+		}
+		if (result == 0 && src_size < (width * height * 10 / 8)) {
+			V4LCONVERT_ERR("short y10b data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_RGB565:
@@ -842,10 +1044,18 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			v4lconvert_rgb565_to_yuv420(src, dest, fmt, 1);
 			break;
 		}
+		if (src_size < (width * height * 2)) {
+			V4LCONVERT_ERR("short rgb565 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_RGB24:
 		switch (dest_pix_fmt) {
+		case V4L2_PIX_FMT_RGB24:
+			memcpy(dest, src, width * height * 3);
+			break;
 		case V4L2_PIX_FMT_BGR24:
 			v4lconvert_swap_rgb(src, dest, width, height);
 			break;
@@ -856,6 +1066,11 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 0, 1);
 			break;
 		}
+		if (src_size < (width * height * 3)) {
+			V4LCONVERT_ERR("short rgb24 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_BGR24:
@@ -863,6 +1078,9 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		case V4L2_PIX_FMT_RGB24:
 			v4lconvert_swap_rgb(src, dest, width, height);
 			break;
+		case V4L2_PIX_FMT_BGR24:
+			memcpy(dest, src, width * height * 3);
+			break;
 		case V4L2_PIX_FMT_YUV420:
 			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 1, 0);
 			break;
@@ -870,6 +1088,11 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			v4lconvert_rgb24_to_yuv420(src, dest, fmt, 1, 1);
 			break;
 		}
+		if (src_size < (width * height * 3)) {
+			V4LCONVERT_ERR("short bgr24 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_YUV420:
@@ -882,10 +1105,18 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 			v4lconvert_yuv420_to_bgr24(src, dest, width,
 					height, 0);
 			break;
+		case V4L2_PIX_FMT_YUV420:
+			memcpy(dest, src, width * height * 3 / 2);
+			break;
 		case V4L2_PIX_FMT_YVU420:
 			v4lconvert_swap_uv(src, dest, fmt);
 			break;
 		}
+		if (src_size < (width * height * 3 / 2)) {
+			V4LCONVERT_ERR("short yuv420 data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_YVU420:
@@ -901,60 +1132,83 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 		case V4L2_PIX_FMT_YUV420:
 			v4lconvert_swap_uv(src, dest, fmt);
 			break;
+		case V4L2_PIX_FMT_YVU420:
+			memcpy(dest, src, width * height * 3 / 2);
+			break;
+		}
+		if (src_size < (width * height * 3 / 2)) {
+			V4LCONVERT_ERR("short yvu420 data frame\n");
+			errno = EPIPE;
+			result = -1;
 		}
 		break;
 
 	case V4L2_PIX_FMT_YUYV:
 		switch (dest_pix_fmt) {
 		case V4L2_PIX_FMT_RGB24:
-			v4lconvert_yuyv_to_rgb24(src, dest, width, height);
+			v4lconvert_yuyv_to_rgb24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_BGR24:
-			v4lconvert_yuyv_to_bgr24(src, dest, width, height);
+			v4lconvert_yuyv_to_bgr24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_YUV420:
-			v4lconvert_yuyv_to_yuv420(src, dest, width, height, 0);
+			v4lconvert_yuyv_to_yuv420(src, dest, width, height, bytesperline, 0);
 			break;
 		case V4L2_PIX_FMT_YVU420:
-			v4lconvert_yuyv_to_yuv420(src, dest, width, height, 1);
+			v4lconvert_yuyv_to_yuv420(src, dest, width, height, bytesperline, 1);
 			break;
 		}
+		if (src_size < (width * height * 2)) {
+			V4LCONVERT_ERR("short yuyv data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_YVYU:
 		switch (dest_pix_fmt) {
 		case V4L2_PIX_FMT_RGB24:
-			v4lconvert_yvyu_to_rgb24(src, dest, width, height);
+			v4lconvert_yvyu_to_rgb24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_BGR24:
-			v4lconvert_yvyu_to_bgr24(src, dest, width, height);
+			v4lconvert_yvyu_to_bgr24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_YUV420:
 			/* Note we use yuyv_to_yuv420 not v4lconvert_yvyu_to_yuv420,
 			   with the last argument reversed to make it have as we want */
-			v4lconvert_yuyv_to_yuv420(src, dest, width, height, 1);
+			v4lconvert_yuyv_to_yuv420(src, dest, width, height, bytesperline, 1);
 			break;
 		case V4L2_PIX_FMT_YVU420:
-			v4lconvert_yuyv_to_yuv420(src, dest, width, height, 0);
+			v4lconvert_yuyv_to_yuv420(src, dest, width, height, bytesperline, 0);
 			break;
 		}
+		if (src_size < (width * height * 2)) {
+			V4LCONVERT_ERR("short yvyu data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	case V4L2_PIX_FMT_UYVY:
 		switch (dest_pix_fmt) {
 		case V4L2_PIX_FMT_RGB24:
-			v4lconvert_uyvy_to_rgb24(src, dest, width, height);
+			v4lconvert_uyvy_to_rgb24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_BGR24:
-			v4lconvert_uyvy_to_bgr24(src, dest, width, height);
+			v4lconvert_uyvy_to_bgr24(src, dest, width, height, bytesperline);
 			break;
 		case V4L2_PIX_FMT_YUV420:
-			v4lconvert_uyvy_to_yuv420(src, dest, width, height, 0);
+			v4lconvert_uyvy_to_yuv420(src, dest, width, height, bytesperline, 0);
 			break;
 		case V4L2_PIX_FMT_YVU420:
-			v4lconvert_uyvy_to_yuv420(src, dest, width, height, 1);
+			v4lconvert_uyvy_to_yuv420(src, dest, width, height, bytesperline, 1);
 			break;
 		}
+		if (src_size < (width * height * 2)) {
+			V4LCONVERT_ERR("short uyvy data frame\n");
+			errno = EPIPE;
+			result = -1;
+		}
 		break;
 
 	default:
@@ -966,7 +1220,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	fmt->fmt.pix.pixelformat = dest_pix_fmt;
 	v4lconvert_fixup_fmt(fmt);
 
-	return 0;
+	return result;
 }
 
 int v4lconvert_convert(struct v4lconvert_data *data,
@@ -1045,7 +1299,13 @@ int v4lconvert_convert(struct v4lconvert_data *data,
 				my_src_fmt.fmt.pix.pixelformat,
 				my_dest_fmt.fmt.pix.pixelformat))
 		convert = 2;
-	else if (my_dest_fmt.fmt.pix.pixelformat != my_src_fmt.fmt.pix.pixelformat)
+	else if (my_dest_fmt.fmt.pix.pixelformat !=
+			my_src_fmt.fmt.pix.pixelformat ||
+		 /* Special case if we do not need to do conversion, but we
+		    are not doing any other step involving copying either,
+		    force going through convert_pixfmt to copy the data from
+		    source to dest */
+		 (!rotate90 && !hflip && !vflip && !crop))
 		convert = 1;
 
 	/* convert_pixfmt (only if convert == 2) -> processing -> convert_pixfmt ->
@@ -1148,7 +1408,8 @@ static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
 
 	for (i = 0; ; i++) {
 		frmsize.index = i;
-		if (SYS_IOCTL(data->fd, VIDIOC_ENUM_FRAMESIZES, &frmsize))
+		if (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_ENUM_FRAMESIZES, &frmsize))
 			break;
 
 		/* We got a framesize, check we don't have the same one already */
@@ -1208,7 +1469,8 @@ int v4lconvert_enum_framesizes(struct v4lconvert_data *data,
 			errno = EINVAL;
 			return -1;
 		}
-		return SYS_IOCTL(data->fd, VIDIOC_ENUM_FRAMESIZES, frmsize);
+		return data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_ENUM_FRAMESIZES, frmsize);
 	}
 
 	if (frmsize->index >= data->no_framesizes) {
@@ -1244,7 +1506,8 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
 			errno = EINVAL;
 			return -1;
 		}
-		res = SYS_IOCTL(data->fd, VIDIOC_ENUM_FRAMEINTERVALS, frmival);
+		res = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_ENUM_FRAMEINTERVALS, frmival);
 		if (res)
 			V4LCONVERT_ERR("%s\n", strerror(errno));
 		return res;
@@ -1289,7 +1552,8 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
 	frmival->pixel_format = src_fmt.fmt.pix.pixelformat;
 	frmival->width = src_fmt.fmt.pix.width;
 	frmival->height = src_fmt.fmt.pix.height;
-	res = SYS_IOCTL(data->fd, VIDIOC_ENUM_FRAMEINTERVALS, frmival);
+	res = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_ENUM_FRAMEINTERVALS, frmival);
 	if (res) {
 		int dest_pixfmt = dest_fmt.fmt.pix.pixelformat;
 		int src_pixfmt  = src_fmt.fmt.pix.pixelformat;
@@ -1331,3 +1595,13 @@ int v4lconvert_vidioc_s_ctrl(struct v4lconvert_data *data, void *arg)
 {
 	return v4lcontrol_vidioc_s_ctrl(data->control, arg);
 }
+
+int v4lconvert_get_fps(struct v4lconvert_data *data)
+{
+	return data->fps;
+}
+
+void v4lconvert_set_fps(struct v4lconvert_data *data, int fps)
+{
+	data->fps = fps;
+}
