@@ -32,11 +32,14 @@
 #include <errno.h>
 #include <signal.h>
 #include <argp.h>
+#include <sys/time.h>
+
+#include <config.h>
 
 #include <linux/dvb/dmx.h>
 #include "dvb-file.h"
 #include "dvb-demux.h"
-#include "libscan.h"
+#include "dvb-scan.h"
 
 #define CHANNEL_FILE	"channels.conf"
 #define PROGRAM_NAME	"dvbv5-zap"
@@ -48,12 +51,17 @@ struct arguments {
 	char *confname, *lnb_name, *output, *demux_dev, *dvr_dev;
 	char *filename;
 	unsigned adapter, frontend, demux, get_detected, get_nit;
-	int lnb, sat_number;
-	unsigned diseqc_wait, silent, frontend_only, freq_bpf;
+	int force_dvbv3, lnb, sat_number;
+	unsigned diseqc_wait, silent, verbose, frontend_only, freq_bpf;
 	unsigned timeout, dvr, rec_psi, exit_after_tuning;
-	unsigned human_readable, record;
+	unsigned record;
 	unsigned n_apid, n_vpid;
 	enum file_formats input_format, output_format;
+	unsigned traffic_monitor, low_traffic;
+	char *search;
+
+	/* Used by status print */
+	unsigned n_status_lines;
 };
 
 static const struct argp_option options[] = {
@@ -65,17 +73,21 @@ static const struct argp_option options[] = {
 	{"lnbf",	'l', "LNBf_type",		0, "type of LNBf to use. 'help' lists the available ones", 0},
 	{"sat_number",	'S', "satellite_number",	0, "satellite number. If not specified, disable DISEqC", 0},
 	{"freq_bpf",	'U', "frequency",		0, "SCR/Unicable band-pass filter frequency to use, in kHz", 0},
-	{"wait",	'W', "time",			0, "adds aditional wait time for DISEqC command completion", 0},
+	{"wait",	'W', "time",			0, "adds additional wait time for DISEqC command completion", 0},
 	{"channels",	'c', "file",			0, "read channels list from 'file'", 0},
 	{"exit",	'x', NULL,			0, "exit after tuning", 0},
 	{"record",	'r', NULL,			0, "set up /dev/dvb/adapterX/dvr0 for TS recording", 0},
 	{"pat",		'p', NULL,			0, "add pat and pmt to TS recording (implies -r)", 0},
 	{"silence",	's', NULL,			0, "increases silence (can be used more than once)", 0},
-	{"human",	'H', NULL,			0, "human readable output", 0},
+	{"verbose",	'v', NULL,			0, "verbose debug messages (can be used more than once)", 0},
 	{"frontend",	'F', NULL,			0, "set up frontend only, don't touch demux", 0},
 	{"timeout",	't', "seconds",			0, "timeout for zapping and for recording", 0},
 	{"output",	'o', "file",			0, "output filename (use -o - for stdout)", 0},
 	{"input-format", 'I',	"format",		0, "Input format: ZAP, CHANNEL, DVBV5 (default: DVBV5)", 0},
+	{"dvbv3",	'3', NULL,			0, "Use DVBv3 only", 0},
+	{"monitor",	'm', NULL,			0, "monitors the DVB traffic", 0},
+	{"low_traffic",	'X', NULL,			0, "also shows DVB traffic with less then 1 packet per second", 0},
+	{"search",	'L', NULL,			0, "search/look for a string inside the traffic", 0},
 	{ 0, 0, 0, 0, 0, 0 }
 };
 
@@ -126,7 +138,7 @@ static int parse(struct arguments *args,
 		sys = SYS_UNDEFINED;
 		break;
 	}
-	dvb_file = read_file_format(args->confname, sys,
+	dvb_file = dvb_read_file_format(args->confname, sys,
 				    args->input_format);
 	if (!dvb_file)
 		return -2;
@@ -170,12 +182,12 @@ static int parse(struct arguments *args,
 			if (type != entry->other_el_pid[i].type) {
 				type = entry->other_el_pid[i].type;
 				if (i)
-					printf("\n");
-				printf("service has pid type %02x: ", type);
+					fprintf(stderr, "\n");
+				fprintf(stderr, "service has pid type %02x: ", type);
 			}
-			printf(" %d", entry->other_el_pid[i].pid);
+			fprintf(stderr, " %d", entry->other_el_pid[i].pid);
 		}
-		printf("\n");
+		fprintf(stderr, "\n");
 	}
 	*sid = entry->service_id;
 
@@ -270,49 +282,74 @@ static void do_timeout(int x)
 	}
 }
 
-static int old_status = 0;
-
-static int print_frontend_stats(struct dvb_v5_fe_parms *parms,
-				int human_readable)
+static int print_frontend_stats(FILE *fd,
+				struct arguments *args,
+				struct dvb_v5_fe_parms *parms)
 {
-	int rc;
-	fe_status_t status;
-	uint32_t snr = 0, _signal = 0;
-	uint32_t ber = 0, uncorrected_blocks = 0;
+	char buf[512], *p;
+	int rc, i, len, show;
+	uint32_t status = 0;
+
+	/* Move cursor up and cleans down */
+	if (isatty(fileno(fd)) && args->n_status_lines)
+		fprintf(fd, "\r\x1b[%dA\x1b[J", args->n_status_lines);
+
+	args->n_status_lines = 0;
 
 	rc = dvb_fe_get_stats(parms);
-	if (rc < 0) {
+	if (rc) {
 		PERROR("dvb_fe_get_stats failed");
 		return -1;
 	}
 
-	rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
-	rc += dvb_fe_retrieve_stats(parms, DTV_BER, &ber);
-	rc += dvb_fe_retrieve_stats(parms, DTV_SIGNAL_STRENGTH, &_signal);
-	rc += dvb_fe_retrieve_stats(parms, DTV_UNCORRECTED_BLOCKS,
-				    &uncorrected_blocks);
-	rc += dvb_fe_retrieve_stats(parms, DTV_SNR, &snr);
+	p = buf;
+	len = sizeof(buf);
+	dvb_fe_snprintf_stat(parms,  DTV_STATUS, NULL, 0, &p, &len, &show);
 
-	if (human_readable) {
-		printf("status %02x | signal %3u%% | snr %3u%% | ber %d | unc %d | ",
-		     status, (_signal * 100) / 0xffff, (snr * 100) / 0xffff,
-		     ber, uncorrected_blocks);
-	} else {
-		fprintf(stderr,
-			"status %02x | signal %04x | snr %04x | ber %08x | unc %08x | ",
-			status, _signal, snr, ber, uncorrected_blocks);
-	}
+	for (i = 0; i < MAX_DTV_STATS; i++) {
+		show = 1;
 
-	if (status & FE_HAS_LOCK) {
-		fprintf(stderr, "FE_HAS_LOCK");
-		if (!(old_status & FE_HAS_LOCK)) {
-			fprintf(stderr, "\n");
-	                dvb_fe_get_parms(parms);
+		dvb_fe_snprintf_stat(parms, DTV_QUALITY, "Quality",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_SIGNAL_STRENGTH, "Signal",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_CNR, "C/N",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_ERROR_BLOCK_COUNT, "UCB",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_BER, "postBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PRE_BER, "preBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PER, "PER",
+				     i,  &p, &len, &show);
+
+		if (p != buf) {
+			if (args->n_status_lines)
+				fprintf(fd, "\t%s\n", buf);
+			else
+				fprintf(fd, "%s\n", buf);
+
+			args->n_status_lines++;
+
+			p = buf;
+			len = sizeof(buf);
 		}
 	}
-	old_status = status;
 
-	fprintf(stderr, "\n");
+	fflush(fd);
+
+	/* While not lock, display status on a new line */
+	dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
+	if (!isatty(fileno(fd)) || !(status & FE_HAS_LOCK))
+		fprintf(fd, "\n");
+
 	return 0;
 }
 
@@ -323,18 +360,18 @@ static int check_frontend(struct arguments *args,
 	fe_status_t status;
 	do {
 		rc = dvb_fe_get_stats(parms);
-		if (rc < 0)
+		if (rc)
 			PERROR("dvb_fe_get_stats failed");
 
 		rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
 		if (!args->silent)
-			print_frontend_stats(parms, args->human_readable);
+			print_frontend_stats(stderr, args, parms);
 		if (args->exit_after_tuning && (status & FE_HAS_LOCK))
 			break;
 		usleep(1000000);
 	} while (!timeout_flag);
 	if (args->silent < 2)
-		print_frontend_stats(parms, args->human_readable);
+		print_frontend_stats(stderr, args, parms);
 
 	return 0;
 }
@@ -349,7 +386,7 @@ static void copy_to_file(int in_fd, int out_fd, int timeout, int silent)
 		r = read(in_fd, buf, BUFLEN);
 		if (r < 0) {
 			if (errno == EOVERFLOW) {
-				printf("buffer overrun\n");
+				fprintf(stderr, "buffer overrun\n");
 				continue;
 			}
 			PERROR("Read failed");
@@ -418,23 +455,168 @@ static error_t parse_opt(int k, char *optarg, struct argp_state *state)
 	case 's':
 		args->silent++;
 		break;
+	case 'v':
+		args->verbose++;
+		break;
 	case 'F':
 		args->frontend_only = 1;
 		break;
-	case 'H':
-		args->human_readable = 1;
-		break;
 	case 'A':
 		args->n_apid = strtoul(optarg, NULL, 0);
 		break;
 	case 'V':
 		args->n_vpid = strtoul(optarg, NULL, 0);
+	case '3':
+		args->force_dvbv3 = 1;
+		break;
+	case 'm':
+		args->traffic_monitor = 1;
+		break;
+	case 'X':
+		args->low_traffic = 1;
+		break;
+	case 'L':
+		args->search = strdup(optarg);
+		break;
 	default:
 		return ARGP_ERR_UNKNOWN;
 	};
 	return 0;
 }
 
+#define BSIZE 188
+
+int do_traffic_monitor(struct arguments *args,
+		    struct dvb_v5_fe_parms *parms)
+{
+	int fd, dvr_fd;
+	long long unsigned pidt[0x2001], wait;
+	int packets = 0;
+	struct timeval startt;
+
+	memset(pidt, 0, sizeof(pidt));
+
+	args->exit_after_tuning = 1;
+	check_frontend(args, parms);
+
+	if ((dvr_fd = open(args->dvr_dev, O_RDONLY)) < 0) {
+		PERROR("failed opening '%s'", args->dvr_dev);
+		return -1;
+	}
+
+	if (ioctl(dvr_fd, DMX_SET_BUFFER_SIZE, 1024 * 1024) == -1)
+		perror("DMX_SET_BUFFER_SIZE failed");
+
+	if ((fd = open(args->demux_dev, O_RDWR)) < 0) {
+		PERROR("failed opening '%s'", args->demux_dev);
+		return -1;
+	}
+
+	if (args->silent < 2)
+		fprintf(stderr, "  dvb_set_pesfilter to 0x2000\n");
+	if (dvb_set_pesfilter(fd, 0x2000, DMX_PES_OTHER,
+			      DMX_OUT_TS_TAP, 0) < 0) {
+		PERROR("couldn't set filter");
+		return -1;
+	}
+
+	gettimeofday(&startt, 0);
+	wait = 1000;
+
+	while (1) {
+		unsigned char buffer[BSIZE];
+		int pid, ok;
+		ssize_t r;
+		if ((r = read(dvr_fd, buffer, BSIZE)) <= 0) {
+			if (errno == EOVERFLOW) {
+				struct timeval now;
+				int diff;
+				gettimeofday(&now, 0);
+				diff =
+				    (now.tv_sec - startt.tv_sec) * 1000 +
+				    (now.tv_usec - startt.tv_usec) / 1000;
+				fprintf(stderr, "%.2fs: buffer overrun\n", diff / 1000.);
+				continue;
+			}
+			perror("read");
+			break;
+		}
+		if (r != BSIZE) {
+			fprintf(stderr, "dvbtraffic: only read %zd bytes\n", r);
+			break;
+		}
+		if (buffer[0] != 0x47) {
+			continue;
+			printf("desync (%x)\n", buffer[0]);
+			while (buffer[0] != 0x47)
+				read(fd, buffer, 1);
+			continue;
+		}
+		ok = 1;
+		pid = ((((unsigned) buffer[1]) << 8) |
+		       ((unsigned) buffer[2])) & 0x1FFF;
+
+		if (args->search) {
+			int i, sl = strlen(args->search);
+			ok = 0;
+			if (pid != 0x1fff) {
+				for (i = 0; i < (188 - sl); ++i) {
+					if (!memcmp(buffer + i, args->search, sl))
+						ok = 1;
+				}
+			}
+		}
+
+		if (ok) {
+			pidt[pid]++;
+			pidt[0x2000]++;
+		}
+
+		packets++;
+
+		if (!(packets & 0xFF)) {
+			struct timeval now;
+			int diff;
+			gettimeofday(&now, 0);
+			diff =
+			    (now.tv_sec - startt.tv_sec) * 1000 +
+			    (now.tv_usec - startt.tv_usec) / 1000;
+			if (diff > wait) {
+				if (isatty(STDOUT_FILENO))
+			                printf("\x1b[1H\x1b[2J");
+
+				args->n_status_lines = 0;
+				printf(" PID          FREQ         SPEED       TOTAL\n");
+				int _pid = 0;
+				for (_pid = 0; _pid < 0x2000; _pid++) {
+					if (pidt[_pid]) {
+						if (!args->low_traffic && (pidt[_pid] * 1000. / diff) < 1)
+							continue;
+						printf("%04x %9.2f p/s %8.1f Kbps ",
+						     _pid,
+						     pidt[_pid] * 1000. / diff,
+						     pidt[_pid] * 1000. / diff * 8 * 188 / 1024);
+						if (pidt[_pid] * 188 / 1024)
+							printf("%8llu KB\n", pidt[_pid] * 188 / 1024);
+						else
+							printf(" %8llu B\n", pidt[_pid] * 188);
+					}
+				}
+				/* 0x2000 is the total traffic */
+				printf("TOT %10.2f p/s %8.1f Kbps %8llu KB\n",
+				     pidt[_pid] * 1000. / diff,
+				     pidt[_pid] * 1000. / diff * 8 * 188 / 1024,
+				     pidt[_pid] * 188 / 1024);
+				printf("\n\n");
+				print_frontend_stats(stdout, args, parms);
+				wait += 1000;
+			}
+		}
+	}
+	close (fd);
+	return 0;
+}
+
 int main(int argc, char **argv)
 {
 	struct arguments args;
@@ -474,7 +656,7 @@ int main(int argc, char **argv)
 	}
 
 	if (args.lnb_name) {
-		lnb = search_lnb(args.lnb_name);
+		lnb = dvb_sat_search_lnb(args.lnb_name);
 		if (lnb < 0) {
 			printf("Please select one of the LNBf's below:\n");
 			print_all_lnb();
@@ -503,13 +685,13 @@ int main(int argc, char **argv)
 			asprintf(&args.confname, "%s/.tzap/%s",
 				homedir, CHANNEL_FILE);
 	}
-	printf("reading channels from file '%s'\n", args.confname);
+	fprintf(stderr, "reading channels from file '%s'\n", args.confname);
 
-	parms = dvb_fe_open(args.adapter, args.frontend, 0, 0);
+	parms = dvb_fe_open(args.adapter, args.frontend, args.verbose, args.force_dvbv3);
 	if (!parms)
 		return -1;
 	if (lnb)
-		parms->lnb = get_lnb(lnb);
+		parms->lnb = dvb_sat_get_lnb(lnb);
 	if (args.sat_number > 0)
 		parms->sat_number = args.sat_number % 3;
 	parms->diseqc_wait = args.diseqc_wait;
@@ -527,6 +709,9 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
+	if (args.traffic_monitor)
+		return do_traffic_monitor(&args, parms);
+
 	if (args.rec_psi) {
 		if (sid < 0) {
 			fprintf(stderr, "Service id 0x%04x was not specified at the file\n",
@@ -544,14 +729,18 @@ int main(int argc, char **argv)
 			perror("opening pat demux failed");
 			return -1;
 		}
-		if (set_pesfilter(pat_fd, 0, DMX_PES_OTHER, args.dvr) < 0)
+		if (dvb_set_pesfilter(pat_fd, 0, DMX_PES_OTHER,
+				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
+				args.dvr ? 64 * 1024 : 0) < 0)
 			return -1;
 
 		if ((pmt_fd = open(args.demux_dev, O_RDWR)) < 0) {
 			perror("opening pmt demux failed");
 			return -1;
 		}
-		if (set_pesfilter(pmt_fd, pmtpid, DMX_PES_OTHER, args.dvr) < 0)
+		if (dvb_set_pesfilter(pmt_fd, pmtpid, DMX_PES_OTHER,
+				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
+				args.dvr ? 64 * 1024 : 0) < 0)
 			return -1;
 	}
 
@@ -562,7 +751,11 @@ int main(int argc, char **argv)
 			PERROR("failed opening '%s'", args.demux_dev);
 			return -1;
 		}
-		if (set_pesfilter(video_fd, vpid, DMX_PES_VIDEO, args.dvr) < 0)
+		if (args.silent < 2)
+			fprintf(stderr, "  dvb_set_pesfilter %d\n", vpid);
+		if (dvb_set_pesfilter(video_fd, vpid, DMX_PES_VIDEO,
+				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
+				args.dvr ? 64 * 1024 : 0) < 0)
 			return -1;
 	}
 
@@ -573,8 +766,11 @@ int main(int argc, char **argv)
 			PERROR("failed opening '%s'", args.demux_dev);
 			return -1;
 		}
-
-		if (set_pesfilter(audio_fd, apid, DMX_PES_AUDIO, args.dvr) < 0)
+		if (args.silent < 2)
+			fprintf(stderr, "  dvb_set_pesfilter %d\n", apid);
+		if (dvb_set_pesfilter(audio_fd, apid, DMX_PES_AUDIO,
+				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
+				args.dvr ? 64 * 1024 : 0) < 0)
 			return -1;
 	}
 
@@ -587,7 +783,10 @@ int main(int argc, char **argv)
 			if (strcmp(args.filename, "-") != 0) {
 				file_fd =
 				    open(args.filename,
-					 O_WRONLY | O_LARGEFILE | O_CREAT,
+#ifdef O_LARGEFILE
+					 O_LARGEFILE |
+#endif
+					 O_WRONLY | O_CREAT,
 					 0644);
 				if (file_fd < 0) {
 					PERROR("open of '%s' failed",
@@ -607,12 +806,12 @@ int main(int argc, char **argv)
 			return -1;
 		}
 		if (args.silent < 2)
-			print_frontend_stats(parms, args.human_readable);
+			print_frontend_stats(stderr, &args, parms);
 
 		copy_to_file(dvr_fd, file_fd, args.timeout, args.silent);
 
 		if (args.silent < 2)
-			print_frontend_stats(parms, args.human_readable);
+			print_frontend_stats(stderr, &args, parms);
 	} else {
 		check_frontend(&args, parms);
 	}
