@@ -12,6 +12,7 @@
    GNU General Public License for more details.
  */
 
+#include <config.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -27,9 +28,6 @@
 
 #include "parse.h"
 
-/* Default place where the keymaps will be stored */
-#define CFGDIR "/etc/rc_keymaps"
-
 struct input_keymap_entry_v2 {
 #define KEYMAP_BY_INDEX	(1 << 0)
 	u_int8_t  flags;
@@ -86,14 +84,16 @@ enum ir_protocols {
 	JVC		= 1 << 3,
 	SONY		= 1 << 4,
 	LIRC		= 1 << 5,
+	SANYO		= 1 << 6,
+	RC_5_SZ		= 1 << 7,
 	OTHER		= 1 << 31,
 };
 
 static int parse_code(char *string)
 {
-	struct parse_key *p;
+	struct parse_event *p;
 
-	for (p = keynames; p->name != NULL; p++) {
+	for (p = key_events; p->name != NULL; p++) {
 		if (!strcasecmp(p->name, string))
 			return p->value;
 	}
@@ -108,9 +108,9 @@ static const char doc[] = "\nAllows get/set IR keycode/scancode tables\n"
 	"\nOn the options bellow, the arguments are:\n"
 	"  DEV      - the /dev/input/event* device to control\n"
 	"  SYSDEV   - the ir class as found at /sys/class/rc\n"
-	"  TABLE    - a file wit a set of scancode=keycode value pairs\n"
+	"  TABLE    - a file with a set of scancode=keycode value pairs\n"
 	"  SCANKEY  - a set of scancode1=keycode1,scancode2=keycode2.. value pairs\n"
-	"  PROTOCOL - protocol name (nec, rc-5, rc-6, other) to be enabled\n"
+	"  PROTOCOL - protocol name (nec, rc-5, rc-6, jvc, sony, sanyo, rc-5-sz, lirc, other) to be enabled\n"
 	"  DELAY    - Delay before repeating a keystroke\n"
 	"  PERIOD   - Period to repeat a keystroke\n"
 	"  CFGFILE  - configuration file that associates a driver/table name with a keymap file\n"
@@ -139,7 +139,7 @@ static const char args_doc[] =
 
 /* Static vars to store the parameters */
 static char *devclass = "rc0";
-static char *devname = NULL;
+static char *devicename = NULL;
 static int readtable = 0;
 static int clear = 0;
 static int debug = 0;
@@ -194,15 +194,16 @@ static error_t parse_keyfile(char *fname, char **table)
 
 	fin = fopen(fname, "r");
 	if (!fin) {
-		perror("opening keycode file");
 		return errno;
 	}
 
 	while (fgets(s, sizeof(s), fin)) {
 		char *p = s;
+
+		line++;
 		while (*p == ' ' || *p == '\t')
 			p++;
-		if (!line && p[0] == '#') {
+		if (line==1 && p[0] == '#') {
 			p++;
 			p = strtok(p, "\n\t =:");
 			do {
@@ -223,6 +224,10 @@ static error_t parse_keyfile(char *fname, char **table)
 							ch_proto |= JVC;
 						else if (!strcasecmp(p,"sony"))
 							ch_proto |= SONY;
+						else if (!strcasecmp(p,"sanyo"))
+							ch_proto |= SANYO;
+						else if (!strcasecmp(p,"rc-5-sz"))
+							ch_proto |= RC_5_SZ;
 						else if (!strcasecmp(p,"other") || !strcasecmp(p,"unknown"))
 							ch_proto |= OTHER;
 						else {
@@ -267,7 +272,7 @@ static error_t parse_keyfile(char *fname, char **table)
 				perror("value");
 		}
 
-		nextkey->codes[0] = (unsigned) strtol(scancode, NULL, 0);
+		nextkey->codes[0] = (unsigned) strtoul(scancode, NULL, 0);
 		nextkey->codes[1] = (unsigned) value;
 		nextkey->next = calloc(1, sizeof(*nextkey));
 		if (!nextkey->next) {
@@ -275,7 +280,6 @@ static error_t parse_keyfile(char *fname, char **table)
 			return ENOMEM;
 		}
 		nextkey = nextkey->next;
-		line++;
 	}
 	fclose(fin);
 
@@ -283,7 +287,7 @@ static error_t parse_keyfile(char *fname, char **table)
 
 err_einval:
 	fprintf(stderr, "Invalid parameter on line %d of %s\n",
-		line + 1, fname);
+		line, fname);
 	return EINVAL;
 
 }
@@ -308,6 +312,8 @@ static error_t parse_cfgfile(char *fname)
 
 	while (fgets(s, sizeof(s), fin)) {
 		char *p = s;
+
+		line++;
 		while (*p == ' ' || *p == '\t')
 			p++;
 
@@ -345,7 +351,6 @@ static error_t parse_cfgfile(char *fname)
 			return ENOMEM;
 		}
 		nextcfg = nextcfg->next;
-		line++;
 	}
 	fclose(fin);
 
@@ -353,7 +358,7 @@ static error_t parse_cfgfile(char *fname)
 
 err_einval:
 	fprintf(stderr, "Invalid parameter on line %d of %s\n",
-		line + 1, fname);
+		line, fname);
 	return EINVAL;
 
 }
@@ -381,7 +386,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		period = atoi(arg);
 		break;
 	case 'd':
-		devname = arg;
+		devicename = arg;
 		break;
 	case 's':
 		devclass = arg;
@@ -410,7 +415,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		do {
 			if (!p)
 				goto err_inval;
-			nextkey->codes[0] = strtol(p, NULL, 0);
+			nextkey->codes[0] = strtoul(p, NULL, 0);
 			if (errno)
 				goto err_inval;
 
@@ -426,7 +431,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			nextkey->codes[1] = key;
 
 			if (debug)
-				fprintf(stderr, "scancode %i=%i\n",
+				fprintf(stderr, "scancode 0x%04x=%u\n",
 					nextkey->codes[0], nextkey->codes[1]);
 
 			nextkey->next = calloc(1, sizeof(keys));
@@ -452,8 +457,12 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 				ch_proto |= JVC;
 			else if (!strcasecmp(p,"sony"))
 				ch_proto |= SONY;
+			else if (!strcasecmp(p,"sanyo"))
+				ch_proto |= SANYO;
 			else if (!strcasecmp(p,"lirc"))
 				ch_proto |= LIRC;
+			else if (!strcasecmp(p,"rc-5-sz"))
+				ch_proto |= RC_5_SZ;
 			else
 				goto err_inval;
 			p = strtok(NULL, ",;");
@@ -479,9 +488,9 @@ static struct argp argp = {
 
 static void prtcode(int *codes)
 {
-	struct parse_key *p;
+	struct parse_event *p;
 
-	for (p = keynames; p->name != NULL; p++) {
+	for (p = key_events; p->name != NULL; p++) {
 		if (p->value == (unsigned)codes[1]) {
 			printf("scancode 0x%04x = %s (0x%02x)\n", codes[0], p->name, codes[1]);
 			return;
@@ -489,9 +498,9 @@ static void prtcode(int *codes)
 	}
 
 	if (isprint (codes[1]))
-		printf("scancode %d = '%c' (0x%02x)\n", codes[0], codes[1], codes[1]);
+		printf("scancode 0x%04x = '%c' (0x%02x)\n", codes[0], codes[1], codes[1]);
 	else
-		printf("scancode %d = 0x%02x\n", codes[0], codes[1]);
+		printf("scancode 0x%04x = 0x%02x\n", codes[0], codes[1]);
 }
 
 static void free_names(struct sysfs_names *names)
@@ -723,6 +732,10 @@ static enum ir_protocols v1_get_hw_protocols(char *name)
 			proto |= JVC;
 		else if (!strcmp(p, "sony"))
 			proto |= SONY;
+		else if (!strcmp(p, "sanyo"))
+			proto |= SANYO;
+		else if (!strcmp(p, "rc-5-sz"))
+			proto |= RC_5_SZ;
 		else
 			proto |= OTHER;
 
@@ -763,6 +776,12 @@ static int v1_set_hw_protocols(struct rc_device *rc_dev)
 	if (rc_dev->current & SONY)
 		fprintf(fp, "sony ");
 
+	if (rc_dev->current & SANYO)
+		fprintf(fp, "sanyo ");
+
+	if (rc_dev->current & RC_5_SZ)
+		fprintf(fp, "rc-5-sz ");
+
 	if (rc_dev->current & OTHER)
 		fprintf(fp, "unknown ");
 
@@ -883,8 +902,12 @@ static enum ir_protocols v2_get_protocols(struct rc_device *rc_dev, char *name)
 			proto = JVC;
 		else if (!strcmp(p, "sony"))
 			proto = SONY;
+		else if (!strcmp(p, "sanyo"))
+			proto = SANYO;
 		else if (!strcmp(p, "lirc"))	/* Only V2 has LIRC support */
 			proto = LIRC;
+		else if (!strcmp(p, "rc-5-sz"))
+			proto = RC_5_SZ;
 		else
 			proto = OTHER;
 
@@ -932,9 +955,15 @@ static int v2_set_protocols(struct rc_device *rc_dev)
 	if (rc_dev->current & SONY)
 		fprintf(fp, "+sony\n");
 
+	if (rc_dev->current & SANYO)
+		fprintf(fp, "+sanyo\n");
+
 	if (rc_dev->current & LIRC)
 		fprintf(fp, "+lirc\n");
 
+	if (rc_dev->current & RC_5_SZ)
+		fprintf(fp, "+rc-5-sz\n");
+
 	if (rc_dev->current & OTHER)
 		fprintf(fp, "+unknown\n");
 
@@ -958,8 +987,12 @@ static void show_proto(	enum ir_protocols proto)
 		fprintf (stderr, "JVC ");
 	if (proto & SONY)
 		fprintf (stderr, "SONY ");
+	if (proto & SANYO)
+		fprintf (stderr, "SANYO ");
 	if (proto & LIRC)
 		fprintf (stderr, "LIRC ");
+	if (proto & RC_5_SZ)
+		fprintf (stderr, "RC-5-SZ ");
 	if (proto & OTHER)
 		fprintf (stderr, "other ");
 }
@@ -1203,6 +1236,18 @@ static void display_proto(struct rc_device *rc_dev)
 	fprintf(stderr, "\n");
 }
 
+
+static char *get_event_name(struct parse_event *event, u_int16_t code)
+{
+	struct parse_event *p;
+
+	for (p = event; p->name != NULL; p++) {
+		if (p->value == code)
+			return p->name;
+	}
+	return "";
+}
+
 static void test_event(int fd)
 {
 	struct input_event ev[64];
@@ -1218,45 +1263,52 @@ static void test_event(int fd)
 		}
 
 		for (i = 0; i < rd / sizeof(struct input_event); i++) {
+			printf("%ld.%06ld: event type %s(0x%02x)",
+				ev[i].time.tv_sec, ev[i].time.tv_usec,
+				get_event_name(events_type, ev[i].type), ev[i].type);
+
 			switch (ev[i].type) {
-			case EV_MSC:
-				if (ev[i].code != MSC_SCAN)
-					break;
-				printf("%ld.%06ld: event MSC: scancode = %02x\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec, ev[i].value);
+			case EV_SYN:
+				printf(".\n");
 				break;
-			case EV_KEY: 			{
-				struct parse_key *p;
-				char *name = "";
-
-				printf("%ld.%06ld: event key %s: ",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
-					(ev[i].value == 0) ? "up" : "down"
-					);
-
-				for (p = keynames; p->name != NULL; p++) {
-					if (p->value == ev[i].code) {
-						name = p->name;
-						break;
-					}
-				}
-				printf("%s (0x%04x)\n", name, ev[i].code);
-
+			case EV_KEY:
+				printf(" key_%s: %s(0x%04x)\n",
+					(ev[i].value == 0) ? "up" : "down",
+					get_event_name(key_events, ev[i].code),
+					ev[i].type);
 				break;
-			}
-			case EV_REP:
-				printf("%ld.%06ld: event repeat: %d\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
+			case EV_REL:
+				printf(": %s (0x%04x) value=%d\n",
+					get_event_name(rel_events, ev[i].code),
+					ev[i].type,
 					ev[i].value);
 				break;
-			case EV_SYN:
-				printf("%ld.%06ld: event sync\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec);
+			case EV_ABS:
+				printf(": %s (0x%04x) value=%d\n",
+					get_event_name(abs_events, ev[i].code),
+					ev[i].type,
+					ev[i].value);
 				break;
+			case EV_MSC:
+				if (ev[i].code == MSC_SCAN)
+					printf(": scancode = 0x%02x\n", ev[i].value);
+				else
+					printf(": code = %s(0x%02x), value = %d\n",
+						get_event_name(msc_events, ev[i].code),
+						ev[i].code, ev[i].value);
+				break;
+			case EV_REP:
+				printf(": value = %d\n", ev[i].value);
+				break;
+			case EV_SW:
+			case EV_LED:
+			case EV_SND:
+			case EV_FF:
+			case EV_PWR:
+			case EV_FF_STATUS:
 			default:
-				printf("%ld.%06ld: event type %d: value: %d\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
-					ev[i].type, ev[i].value);
+				printf(": code = 0x%02x, value = %d\n",
+					ev[i].code, ev[i].value);
 				break;
 			}
 		}
@@ -1298,7 +1350,7 @@ static void display_table_v2(struct rc_device *rc_dev, int fd)
 			break;
 
 		/* FIXME: Extend it to support scancodes > 32 bits */
-		codes[0] = ((u_int32_t *)entry.scancode)[0];
+		memcpy(&codes[0], entry.scancode, sizeof(codes[0]));
 		codes[1] = entry.keycode;
 
 		prtcode(codes);
@@ -1350,6 +1402,27 @@ static void show_evdev_attribs(int fd)
 	get_rate(fd, &delay, &period);
 }
 
+static void device_info(int fd, char *prepend)
+{
+	struct input_id id;
+	char buf[32];
+	int rc;
+
+	rc = ioctl(fd, EVIOCGNAME(sizeof(buf)), buf);
+	if (rc >= 0)
+		fprintf(stderr,"%sName: %.*s\n",prepend, rc, buf);
+	else
+		perror ("EVIOCGNAME");
+
+	rc = ioctl(fd, EVIOCGID, &id);
+	if (rc >= 0)
+		fprintf(stderr,
+			"%sbus: %d, vendor/product: %04x:%04x, version: 0x%04x\n",
+			prepend, id.bustype, id.vendor, id.product, id.version);
+	else
+		perror ("EVIOCGID");
+}
+
 static int show_sysfs_attribs(struct rc_device *rc_dev)
 {
 	static struct sysfs_names *names, *cur;
@@ -1374,6 +1447,7 @@ static int show_sysfs_attribs(struct rc_device *rc_dev)
 			display_proto(rc_dev);
 			fd = open(rc_dev->input_name, O_RDONLY);
 			if (fd > 0) {
+				device_info(fd, "\t");
 				show_evdev_attribs(fd);
 				close(fd);
 			} else {
@@ -1395,17 +1469,27 @@ int main(int argc, char *argv[])
 
 	/* Just list all devices */
 	if (!clear && !readtable && !keys.next && !ch_proto && !cfg.next && !test && !delay && !period) {
+		if (devicename) {
+			fd = open(devicename, O_RDONLY);
+			if (fd < 0) {
+				perror("Can't open device");
+				return -1;
+			}
+			device_info(fd, "");
+			close(fd);
+			return 0;
+		}
 		if (show_sysfs_attribs(&rc_dev))
 			return -1;
 
 		return 0;
 	}
 
-	if (cfg.next && (clear || keys.next || ch_proto || devname)) {
+	if (cfg.next && (clear || keys.next || ch_proto || devicename)) {
 		fprintf (stderr, "Auto-mode can be used only with --read, --debug and --sysdev options\n");
 		return -1;
 	}
-	if (!devname) {
+	if (!devicename) {
 		names = find_device(devclass);
 		if (!names)
 			return -1;
@@ -1417,7 +1501,7 @@ int main(int argc, char *argv[])
 		names->name = NULL;
 		free_names(names);
 
-		devname = rc_dev.input_name;
+		devicename = rc_dev.input_name;
 		dev_from_class++;
 	}
 
@@ -1427,9 +1511,9 @@ int main(int argc, char *argv[])
 		int rc;
 
 		for (cur = &cfg; cur->next; cur = cur->next) {
-			if (strcasecmp(cur->driver, rc_dev.drv_name) && strcasecmp(cur->driver, "*"))
+			if ((!rc_dev.drv_name || strcasecmp(cur->driver, rc_dev.drv_name)) && strcasecmp(cur->driver, "*"))
 				continue;
-			if (strcasecmp(cur->table, rc_dev.keytable_name) && strcasecmp(cur->table, "*"))
+			if ((!rc_dev.keytable_name || strcasecmp(cur->table, rc_dev.keytable_name)) && strcasecmp(cur->table, "*"))
 				continue;
 			break;
 		}
@@ -1446,29 +1530,45 @@ int main(int argc, char *argv[])
 				cur->fname);
 		if (cur->fname[0] == '/' || ((cur->fname[0] == '.') && strchr(cur->fname, '/'))) {
 			fname = cur->fname;
+			rc = parse_keyfile(fname, &name);
+			if (rc < 0) {
+				fprintf(stderr, "Can't load %s table\n", fname);
+				return -1;
+			}
 		} else {
-			fname = malloc(strlen(cur->fname) + strlen(CFGDIR) + 2);
-			strcpy(fname, CFGDIR);
+			fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_USER_DIR) + 2);
+			strcpy(fname, IR_KEYTABLE_USER_DIR);
 			strcat(fname, "/");
 			strcat(fname, cur->fname);
+			rc = parse_keyfile(fname, &name);
+			if (rc != 0) {
+				fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_SYSTEM_DIR) + 2);
+				strcpy(fname, IR_KEYTABLE_SYSTEM_DIR);
+				strcat(fname, "/");
+				strcat(fname, cur->fname);
+				rc = parse_keyfile(fname, &name);
+			}
+			if (rc != 0) {
+				fprintf(stderr, "Can't load %s table from %s or %s\n", cur->fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
+				return -1;
+			}
 		}
-		rc = parse_keyfile(fname, &name);
-		if (rc < 0 || !keys.next) {
-			fprintf(stderr, "Can't load %s table or empty table\n", fname);
+		if (!keys.next) {
+			fprintf(stderr, "Empty table %s\n", fname);
 			return -1;
 		}
 		clear = 1;
 	}
 
 	if (debug)
-		fprintf(stderr, "Opening %s\n", devname);
-	fd = open(devname, O_RDONLY);
+		fprintf(stderr, "Opening %s\n", devicename);
+	fd = open(devicename, O_RDONLY);
 	if (fd < 0) {
-		perror(devname);
+		perror(devicename);
 		return -1;
 	}
 	if (dev_from_class)
-		free(devname);
+		free(devicename);
 	if (get_input_protocol_version(fd))
 		return -1;
 
