@@ -16,10 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-
+#include <stdio.h>
 #include <QLabel>
 #include <QImage>
-#include <QTimer>
 #include <QVBoxLayout>
 #include <QCloseEvent>
 
@@ -30,40 +29,18 @@ CaptureWin::CaptureWin()
 {
 	QVBoxLayout *vbox = new QVBoxLayout(this);
 
-	m_frame = 0;
+	setWindowTitle("V4L2 Capture");
 	m_label = new QLabel();
 	m_msg = new QLabel("No frame");
-	m_timer = new QTimer(this);
-	connect(m_timer, SIGNAL(timeout()), this, SLOT(update()));
 
 	vbox->addWidget(m_label);
 	vbox->addWidget(m_msg);
 }
 
-void CaptureWin::setImage(const QImage &image, bool init)
+void CaptureWin::setImage(const QImage &image, const QString &status)
 {
 	m_label->setPixmap(QPixmap::fromImage(image));
-	if (init) {
-		m_frame = m_lastFrame = m_fps = 0;
-		m_msg->setText("No frame");
-	} else {
-		if (m_frame == 0) {
-			m_timer->start(2000);
-		}
-		m_msg->setText(QString("Frame: %1 Fps: %2")
-				.arg(++m_frame).arg(m_fps));
-	}
-}
-
-void CaptureWin::stop()
-{
-	m_timer->stop();
-}
-
-void CaptureWin::update()
-{
-	m_fps = (m_frame - m_lastFrame + 1) / 2;
-	m_lastFrame = m_frame;
+	m_msg->setText(status);
 }
 
 void CaptureWin::closeEvent(QCloseEvent *event)
